==> UNOOutOfBoundAccessCheckModule-1.0.0-1.tar.gz: OutOfBoundAccess/uno.c <==
/***** uno: uno.c *****/

/* Copyright (c) 2000-2003 by Lucent Technologies - Bell Laboratories     */
/* All Rights Reserved.  This software is for educational purposes only.  */
/* Permission is given to distribute this code provided that this intro-  */
/* ductory message is not removed and no monies are exchanged.            */
/* No guarantee is expressed or implied by the distribution of this code. */
/* Software written by Gerard J. Holzmann  --  http://spinroot.com/gerard */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

// #include "PStringToken.h"
#include "uno.h"
// #include "UNOOutOfBoundAccessInfo.h"
// #include "UNOOutOfBoundAccess.h"

/* extern void CreateTokenBuf( char* pText , int nLength );
extern void DeleteTokenBuf();

extern char* GetFirstToken( const char* pSeps );
extern char* GetNextToken( const char* pSeps ); */

// #include "PStringToken.h"

#ifdef DEBUG
	void System( char *cmd )	{ PrintfEx("<%s>\n", cmd); }
#else
	//#define System(x)	{ if (verbose) PrintfEx("<%s>\n", x); system(x); }
//#endif
	void System( char *x )
	{ 
		if (verbose)
		{
			PrintfEx("<%s>\n", x); 
		}
		else
		{
			system( x ); 
		}
	}

	void SystemFunc( char** cmd , int nArgCount , unsigned char bIsGlobal )
	{ 
		if (verbose) 
		{
			PrintfEx("<%s>\n", cmd); 
		}
		else
		{
			if( bIsGlobal )
			{
				GlobalCheck( nArgCount , cmd );
			}
			else
			{
				LocalCheck( nArgCount , cmd );
			}
		}
	}
#endif

void RightTrim( char* pCommand )
{
	int i = 0;
	int j = 0;
	int nLength = 0;

	nLength = strlen( pCommand ) - 1;

	while( isspace( *( pCommand + ( nLength - i ) ) ) )
	{
		i++;
	}

	*( pCommand + ( ( nLength - i ) + 1 ) ) = '\0';
}

/* void LightTrim( char* pCommand )
{
	int i = 0;
	int j = 0;
	
	while( isspace( *( pCommand + ( nLength + i ) ) ) )
	{
		i++;
	}

	*( pCommand + ( nLength - ( i + 1 ) ) ) = '\0';
} */

void CreateTokenBuf( char* pText , int nLength )
{
	int nStringLength = 0;
	
	// memcpy( m_pText , pText , nLength );
	// m_strText = strText
	// strText.TrimRight( " " );
	// nStringLength = strText.GetLength();

	// m_pBuf = new char[ nLength + 1 ];
	m_pBuf = ( char* )malloc( ( sizeof( char ) * ( nLength + 1 ) ) );
	memset( m_pBuf , 0 , nLength + 1 );
	memcpy( m_pBuf , pText , nLength );
}

char* GetFirstToken( const char* pSeps )
{
	char* pData = NULL;

	pData = strtok( m_pBuf , pSeps );
	return pData;			
}

char* GetNextToken( const char* pSeps )
{
	char* pData = NULL;
	
	m_pData = strtok( NULL, pSeps );
	return m_pData;
}

void DeleteTokenBuf()
{
	if( m_pBuf )
	{
		// delete [] m_pBuf;
		free( m_pBuf );
		m_pBuf = NULL;
	}
}

int GetWordCount( char* pBuf , int* nBufLength )
{
	int nCount = 0;
	int i = 0;

	for( ; '\0' != pBuf[ i ] ; i++ )
	{
		if( pBuf[ i ] == ' ' )
		{
			nCount++;
		}
		*nBufLength = ( *nBufLength ) + 1;
	}

	return nCount + 1;
}

char** CreateArgument( char* pBuf , int* nArgCount )
{
	char** pArgBuf = NULL;
	char* pTemp = NULL;

	int nBufLength = 0;
	// int nArgCount = 0;
	int i = 0;
	
	RightTrim( pBuf );

	*nArgCount = GetWordCount( pBuf , &nBufLength );

	CreateTokenBuf( pBuf , nBufLength );
	pTemp = GetFirstToken( " " );

	pArgBuf = ( char** )malloc( sizeof( char* ) * ( *nArgCount ) );

	for( i = 0 ; i < ( *nArgCount ) ; i++ )
	{
		pArgBuf[ i ] = ( char* )malloc( strlen( pTemp ) + 1 );
		// PrintfEx( "strlen( pTemp ) = %d\n" , strlen( pTemp ) );
		memset( pArgBuf[ i ] , 0 , strlen( pTemp ) + 1 );

		memcpy( pArgBuf[ i ] , pTemp , strlen( pTemp ) + 1 );

		pTemp = GetNextToken( " " );
	}

	DeleteTokenBuf();
	
	return pArgBuf; 
}

static void
uno_usage(void)
{	/* duplicates function of /v/bin/uno shellscript in c-program */

	PrintfEx( "usage: uno [options] *.c");
	/* PrintfEx( "usage: uno [options] *.c\n");
	PrintfEx( "uno options:\n");
	PrintfEx( "\t-CPP=x	set preprocessor to x\n");
	PrintfEx( "\t-Dname=def	define    compiler directive\n");
	PrintfEx( "\t-Dname    	define    compiler directive\n");
	PrintfEx( "\t-Uname    	undefine  compiler directive\n");
	PrintfEx( "\t-Iname    	add name to list of dirs searched for include-files\n");
	PrintfEx( "\t-Wname       use name as working directory to store .uno files\n\n");
	PrintfEx( "\t-n		ignore preprocessing directives in source files\n");
	PrintfEx( "\t-o arg	ignored, for modest compatability with cc arguments\n");
	PrintfEx( "\t-m uno.dfn	use master (type) definitions file uno.dfn\n");
	PrintfEx( "\t-x f		declare f to be a function that does not return\n\n");
	PrintfEx( "\t-V        	print version number and exit\n");
	PrintfEx( "\t-s		print symbol table information and exit\n\n");
	PrintfEx( "\t-l or -c	perform only local analysis, not global\n");
	PrintfEx( "\t-p x		check local property def stored in file x\n");
	PrintfEx( "\t-g x		check global property def stored in file x\n\n");
	PrintfEx( "\t-a		report all error paths (local analysis)\n");
	PrintfEx( "\t-t		more detailed execution traces (global analysis)\n");
	PrintfEx( "\t-u		complain about redundancies of all sorts\n");
	PrintfEx( "\t-v		verbose mode (mostly for debugging)\n");
	PrintfEx( "\t-w		more picky, includes -u and -t\n"); */
	ExitEx(1);
}

static void
add_target(char *f)
{	Fnm *n;

	if (f[0] == '_')
		return;	/* avoid re-preprocessing stale uno intermediaries */

	n = (Fnm *) malloc(sizeof(Fnm));
	n->f = (char *) malloc(strlen(f) + 1);
	strcpy(n->f, f);
	n->nxt = fnames;
	fnames = n;
}

static void
pass_loc(char *pref, char *par)
{
	strcat(loc_args, pref);
	if (par)
	{	
		strcat(loc_args, par);
		strcat(loc_args, " ");
	}
}

static void
set_glb(char *par)
{
	glob_prop++;
#ifndef DEBUG
	int fd;
	fd = creat("_uno_.c", 0644);
	if (fd < 0)
	{	PrintfEx( "uno: cannot create temp files\n");
		ExitEx(1);
	}
	close(fd);
#else
	System("echo \"/* empty file */\" > _uno_.c");
#endif
	sprintf(buf, "%s%s -prop %s _uno_.c", BINDIR, LX, par);
	System(buf);
#if 1
	unlink("_uno_.c");
#else
	System("rm -f _uno_.c");
#endif
}

static void
cleanup(int unused)
{	
	int i;

	if (verbose) 
	{
		return;
	}

	if ((int) strlen(glob_cmd) > glob_base)
	{	for (i = 0; i < glob_base; i++)
			glob_cmd[i] = ' ';
#ifdef PC_
		glob_cmd[0] = 'd';
		glob_cmd[1] = 'e';
		glob_cmd[2] = 'l';
#else
		glob_cmd[0] = 'r';
		glob_cmd[1] = 'm';
		glob_cmd[2] = ' ';
		glob_cmd[3] = '-';
		glob_cmd[4] = 'f';
#endif
		if (w_dir)
		{	
			char *p = ( char* )malloc(strlen(glob_cmd) + strlen("cd ;") + strlen(w_dir) + 1);
			sprintf(p, "cd %s; %s\n", w_dir, glob_cmd);
			System(p);
		} 
		else
		{	
			System(glob_cmd);
		}	
	}
}

static void
version_info(void)
{	
	strcat(loc_args, "-V");
	System(loc_args);
	ExitEx(0);
}

static void
run_uno(void)
{	
	char *p;
	char** ppArg = NULL;
	int nArgCount = 0;

	while (fnames)
	{	strcpy(buf, loc_args);
		strcat(buf, fnames->f);

#ifdef DEBUG
		System(buf);
#else
		//   3 ΰ~  ϴ Լ д, exit Լ    Լ ȣ
		ppArg  = CreateArgument( buf , &nArgCount );
		SystemFunc( ppArg , nArgCount , 0 );
		FreeArgument( &ppArg , nArgCount );
#endif

		strcat(glob_cmd, fnames->f);
		p = strrchr(glob_cmd, '.');
		if (!p)
		{	PrintfEx( "uno: cannot happen1\n");
			ExitEx(1);
		}
		*p = '\0'; /* replace .c with .uno */
		strcat(glob_cmd, ".uno ");

		fnames = fnames->nxt;
	}
	if (!localonly)
	{
		if (glob_prop)
		{
			strcat(glob_cmd, "_uno_.uno ");
		}
		if (w_dir)
		{	
			p = ( char* )malloc(strlen(glob_cmd) + strlen("cd ;") + strlen(w_dir) + 1);
			sprintf(p, "cd %s; %s\n", w_dir, glob_cmd);
			System(p);
		} 
		else
		{	
#ifdef DEBUG
		System(buf);
#else
			/* if( 1 == g_nCheckOption )
			{
				ppArg  = CreateArgument( glob_cmd , &nArgCount );
				SystemFunc( ppArg , nArgCount , 1 );
				FreeArgument( &ppArg , nArgCount );
			} */
#endif
		}	
	}
	cleanup(0);
}

int UnoCheck( char* pBuffer )
{
	// pBuffer  ̽ Ľؼ UnoCheckEx Ѱش.
	return 0;
}

int UnoCheckEx(int argc, char *argv[] , treenode* pParseTree , char* pszFileName )
{
	char aArray[ 3 ][24] = { "CheckBound" , "CheckUninitialize" , "CheckMemory" };
	// fprintf( stdout , "OutOfBound Module Id %d" , MODULE_ID );
	// fflush( stdout );

	g_pParseTree = pParseTree;
	g_pszFileName = pszFileName;

	if( !strncmp( *argv , aArray[ 0 ] , strlen( aArray[ 0 ] ) ) )
	{
		g_nCheckOption = 0;
	}
	else if( !strncmp( *argv , aArray[ 1 ] , strlen( aArray[ 1 ] ) ) )
	{
		g_nCheckOption = 1;
	}
	else
	{			
		g_nCheckOption = 2;
	}

	strcpy(loc_args, BINDIR);
	strcat(loc_args, LX);
	strcat(loc_args, " ");

	strcpy(glob_cmd, BINDIR);
	strcat(glob_cmd, GX);
	strcat(glob_cmd, " ");

	argc--;
	while (argc-- > 0)	/* @uno_suppress side-effect */
	{	
		argv++;
		if (0) PrintfEx("%3d: '%s'\n", argc, *argv);
		if ((*argv)[0] == '-')
		{	if (0) PrintfEx("option '%c' -- '%s'\n", (*argv)[1], *argv);
			switch((*argv)[1]) {
			case 'C':	/* -CPP=... */
			case 'D':
			case 'U':
			case 'I':
				pass_loc(*argv, "");
				break;

			case 'W':
				w_dir = ( char* )malloc(strlen(*argv)+1-2);
				strcpy(w_dir, &(*argv)[2]);
				pass_loc(*argv, "");
				break;
			case 'a':	/* l */
				pass_loc("-allerr ", NULL);
				break;
			case 'g':	/* g */
				argv++; argc--;
				set_glb(*argv);
				break;
			case 'c':	/* absorb cc args */
			case 'l':	/* l */
				localonly = 1;
				pass_loc("-localonly ", NULL);
				break;
			case 'm':	/* l */
				argv++; argc--;
				pass_loc("-master ", *argv);
				break;
			case 'n':	/* l */
				pass_loc("-nopre ", NULL);
				break;
			case 'o':	/* absorb cc args */
				argv++; argc--;
				break;
			case 'p':	/* l */
				argv++; argc--;
				pass_loc("-prop ", *argv);
				break;
			case 's':	/* l */
				pass_loc("-s ", NULL);
				localonly = 1;
				break;
			case 't':	/* g */
				strcat(glob_cmd, "-l ");
				break;	/* g */
			case 'V':	/* l */
				version_info();
				break;
			case 'v':	/* g,l */
				pass_loc("-v ", NULL);
				strcat(glob_cmd, "-v ");
				verbose = 1;
				break;
			case 'w':	/* g,l */
				pass_loc("-picky ", NULL);
				strcat(glob_cmd, "-l ");
			case 'u':	/* g,l  @uno_suppress: fall through on switch */
				usecheck = 1;
				pass_loc("-use ", NULL);
				strcat(glob_cmd, "-u ");
				break;
			case 'x':	/* l */
				argv++; argc--;
				pass_loc("-exit ", *argv);
				break;
			default :
				uno_usage();
				break;
			}
		} 
		else if (strstr(*argv, ".c") != NULL)
		{
			add_target(*argv);
		}
		else
		{
			uno_usage();
		}

		/*
 		if( 0 == argc )
		{
			break;
		}

		argv++; 
		*/
	}
	if (!fnames)
		uno_usage();

#ifndef PC
	signal(SIGINT, cleanup);
#endif

	glob_base = (int) strlen(glob_cmd);

	run_uno();

	if (!usecheck)
	// PrintfEx("uno:\tcheck completed, try 'uno -h' for different checks\n");

	return 0;
}

==> UNOOutOfBoundAccessCheckModule-1.1.1-1.tar.gz: OutOfBoundAccess/uno.c <==
/***** uno: uno.c *****/

/* Copyright (c) 2000-2003 by Lucent Technologies - Bell Laboratories     */
/* All Rights Reserved.  This software is for educational purposes only.  */
/* Permission is given to distribute this code provided that this intro-  */
/* ductory message is not removed and no monies are exchanged.            */
/* No guarantee is expressed or implied by the distribution of this code. */
/* Software written by Gerard J. Holzmann  --  http://spinroot.com/gerard */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>

// #include "PStringToken.h"
#include "uno.h"
// #include "UNOOutOfBoundAccessInfo.h"
// #include "UNOOutOfBoundAccess.h"

/* extern void CreateTokenBuf( char* pText , int nLength );
extern void DeleteTokenBuf();

extern char* GetFirstToken( const char* pSeps );
extern char* GetNextToken( const char* pSeps ); */

// #include "PStringToken.h"

#ifdef DEBUG
	void System( char *cmd )	{ PrintfEx("<%s>\n", cmd); }
#else
	//#define System(x)	{ if (verbose) PrintfEx("<%s>\n", x); system(x); }
//#endif
	void System( char *x )
	{ 
		if (verbose)
		{
			PrintfEx("<%s>\n", x); 
		}
		else
		{
			system( x ); 
		}
	}

	void SystemFunc( char** cmd , int nArgCount , unsigned char bIsGlobal )
	{ 
		if (verbose) 
		{
			PrintfEx("<%s>\n", cmd); 
		}
		else
		{
			if( bIsGlobal )
			{
				GlobalCheck( nArgCount , cmd );
			}
			else
			{
				LocalCheck( nArgCount , cmd );
			}
		}
	}
#endif

void RightTrim( char* pCommand )
{
	int i = 0;
	int j = 0;
	int nLength = 0;

	nLength = strlen( pCommand ) - 1;

	while( isspace( *( pCommand + ( nLength - i ) ) ) )
	{
		i++;
	}

	*( pCommand + ( ( nLength - i ) + 1 ) ) = '\0';
}

/* void LightTrim( char* pCommand )
{
	int i = 0;
	int j = 0;
	
	while( isspace( *( pCommand + ( nLength + i ) ) ) )
	{
		i++;
	}

	*( pCommand + ( nLength - ( i + 1 ) ) ) = '\0';
} */

void CreateTokenBuf( char* pText , int nLength )
{
	int nStringLength = 0;
	
	// memcpy( m_pText , pText , nLength );
	// m_strText = strText
	// strText.TrimRight( " " );
	// nStringLength = strText.GetLength();

	// m_pBuf = new char[ nLength + 1 ];
	m_pBuf = ( char* )malloc( ( sizeof( char ) * ( nLength + 1 ) ) );
	memset( m_pBuf , 0 , nLength + 1 );
	memcpy( m_pBuf , pText , nLength );
}

char* GetFirstToken( const char* pSeps )
{
	char* pData = NULL;

	pData = strtok( m_pBuf , pSeps );
	return pData;			
}

char* GetNextToken( const char* pSeps )
{
	char* pData = NULL;
	
	m_pData = strtok( NULL, pSeps );
	return m_pData;
}

void DeleteTokenBuf()
{
	if( m_pBuf )
	{
		// delete [] m_pBuf;
		free( m_pBuf );
		m_pBuf = NULL;
	}
}

int GetWordCount( char* pBuf , int* nBufLength )
{
	int nCount = 0;
	int i = 0;

	for( ; '\0' != pBuf[ i ] ; i++ )
	{
		if( pBuf[ i ] == ' ' )
		{
			nCount++;
		}
		*nBufLength = ( *nBufLength ) + 1;
	}

	return nCount + 1;
}

char** CreateArgument( char* pBuf , int* nArgCount )
{
	char** pArgBuf = NULL;
	char* pTemp = NULL;

	int nBufLength = 0;
	// int nArgCount = 0;
	int i = 0;
	
	RightTrim( pBuf );

	*nArgCount = GetWordCount( pBuf , &nBufLength );

	CreateTokenBuf( pBuf , nBufLength );
	pTemp = GetFirstToken( " " );

	pArgBuf = ( char** )malloc( sizeof( char* ) * ( *nArgCount ) );

	for( i = 0 ; i < ( *nArgCount ) ; i++ )
	{
		pArgBuf[ i ] = ( char* )malloc( strlen( pTemp ) + 1 );
		// PrintfEx( "strlen( pTemp ) = %d\n" , strlen( pTemp ) );
		memset( pArgBuf[ i ] , 0 , strlen( pTemp ) + 1 );

		memcpy( pArgBuf[ i ] , pTemp , strlen( pTemp ) + 1 );

		pTemp = GetNextToken( " " );
	}

	DeleteTokenBuf();
	
	return pArgBuf; 
}

static void
uno_usage(void)
{	/* duplicates function of /v/bin/uno shellscript in c-program */

	PrintfEx( "usage: uno [options] *.c");
	/* PrintfEx( "usage: uno [options] *.c\n");
	PrintfEx( "uno options:\n");
	PrintfEx( "\t-CPP=x	set preprocessor to x\n");
	PrintfEx( "\t-Dname=def	define    compiler directive\n");
	PrintfEx( "\t-Dname    	define    compiler directive\n");
	PrintfEx( "\t-Uname    	undefine  compiler directive\n");
	PrintfEx( "\t-Iname    	add name to list of dirs searched for include-files\n");
	PrintfEx( "\t-Wname       use name as working directory to store .uno files\n\n");
	PrintfEx( "\t-n		ignore preprocessing directives in source files\n");
	PrintfEx( "\t-o arg	ignored, for modest compatability with cc arguments\n");
	PrintfEx( "\t-m uno.dfn	use master (type) definitions file uno.dfn\n");
	PrintfEx( "\t-x f		declare f to be a function that does not return\n\n");
	PrintfEx( "\t-V        	print version number and exit\n");
	PrintfEx( "\t-s		print symbol table information and exit\n\n");
	PrintfEx( "\t-l or -c	perform only local analysis, not global\n");
	PrintfEx( "\t-p x		check local property def stored in file x\n");
	PrintfEx( "\t-g x		check global property def stored in file x\n\n");
	PrintfEx( "\t-a		report all error paths (local analysis)\n");
	PrintfEx( "\t-t		more detailed execution traces (global analysis)\n");
	PrintfEx( "\t-u		complain about redundancies of all sorts\n");
	PrintfEx( "\t-v		verbose mode (mostly for debugging)\n");
	PrintfEx( "\t-w		more picky, includes -u and -t\n"); */
	ExitEx(1);
}

static void
add_target(char *f)
{	Fnm *n;

	if (f[0] == '_')
		return;	/* avoid re-preprocessing stale uno intermediaries */

	n = (Fnm *) malloc(sizeof(Fnm));
	n->f = (char *) malloc(strlen(f) + 1);
	strcpy(n->f, f);
	n->nxt = fnames;
	fnames = n;
}

static void
pass_loc(char *pref, char *par)
{
	strcat(loc_args, pref);
	if (par)
	{	
		strcat(loc_args, par);
		strcat(loc_args, " ");
	}
}

static void
set_glb(char *par)
{
	printf( "\n\nset_glb\n\n" );
	glob_prop++;
#ifndef DEBUG
	int fd;
	fd = creat("_uno_.c", 0644);
	if (fd < 0)
	{	PrintfEx( "uno: cannot create temp files\n");
		ExitEx(1);
	}
	close(fd);
#else
	System("echo \"/* empty file */\" > _uno_.c");
#endif
	sprintf(buf, "%s%s -prop %s _uno_.c", BINDIR, LX, par);
	System(buf);
#if 1
	unlink("_uno_.c");
#else
	System("rm -f _uno_.c");
#endif
}

/* static void
cleanup(int unused)
{	
	int i;

	if (verbose) 
	{
		return;
	}

	if ((int) strlen(glob_cmd) > glob_base)
	{	for (i = 0; i < glob_base; i++)
			glob_cmd[i] = ' ';
#ifdef PC_
		glob_cmd[0] = 'd';
		glob_cmd[1] = 'e';
		glob_cmd[2] = 'l';
#else
		glob_cmd[0] = 'r';
		glob_cmd[1] = 'm';
		glob_cmd[2] = ' ';
		glob_cmd[3] = '-';
		glob_cmd[4] = 'f';
#endif
		if (w_dir)
		{	
			char *p = ( char* )malloc(strlen(glob_cmd) + strlen("cd ;") + strlen(w_dir) + 1);
			sprintf(p, "cd %s; %s\n", w_dir, glob_cmd);
			System(p);
		} 
		else
		{	
			System(glob_cmd);
		}	
	}
} */

static void
version_info(void)
{	
	strcat(loc_args, "-V");
	System(loc_args);
	ExitEx(0);
}

static void
run_uno(void)
{	
	char *p;
	char** ppArg = NULL;
	int nArgCount = 0;

	while (fnames)
	{	strcpy(buf, loc_args);
		strcat(buf, fnames->f);

#ifdef DEBUG
		System(buf);
#else
		//   3 ΰ~  ϴ Լ д, exit Լ    Լ ȣ
		ppArg  = CreateArgument( buf , &nArgCount );
		SystemFunc( ppArg , nArgCount , 0 );
		FreeArgument( &ppArg , nArgCount );
#endif

		strcat(glob_cmd, fnames->f);
		p = strrchr(glob_cmd, '.');
		if (!p)
		{	PrintfEx( "uno: cannot happen1\n");
			ExitEx(1);
		}
		*p = '\0'; /* replace .c with .uno */
		strcat(glob_cmd, ".uno ");

		fnames = fnames->nxt;
	}
	if (!localonly)
	{
		if (glob_prop)
		{
			strcat(glob_cmd, "_uno_.uno ");
		}
		if (w_dir)
		{	
			p = ( char* )malloc(strlen(glob_cmd) + strlen("cd ;") + strlen(w_dir) + 1);
			sprintf(p, "cd %s; %s\n", w_dir, glob_cmd);
			System(p);
		} 
		else
		{	
#ifdef DEBUG
		System(buf);
#else
			/* if( 1 == g_nCheckOption )
			{
				ppArg  = CreateArgument( glob_cmd , &nArgCount );
				SystemFunc( ppArg , nArgCount , 1 );
				FreeArgument( &ppArg , nArgCount );
			} */
#endif
		}	
	}
	// cleanup(0);
}

int UnoCheck( char* pBuffer )
{
	// pBuffer  ̽ Ľؼ UnoCheckEx Ѱش.
	return 0;
}

int UnoCheckEx(int argc, char *argv[] , treenode* pParseTree , char* pszFileName )
{
	char aArray[ 3 ][24] = { "CheckBound" , "CheckUninitialize" , "CheckMemory" };
	// fprintf( stdout , "OutOfBound Module Id %d" , MODULE_ID );
	// fflush( stdout );

	g_pParseTree = pParseTree;
	g_pszFileName = pszFileName;

	if( !strncmp( *argv , aArray[ 0 ] , strlen( aArray[ 0 ] ) ) )
	{
		g_nCheckOption = 0;
	}
	else if( !strncmp( *argv , aArray[ 1 ] , strlen( aArray[ 1 ] ) ) )
	{
		g_nCheckOption = 1;
	}
	else
	{			
		g_nCheckOption = 2;
	}

	strcpy(loc_args, BINDIR);
	strcat(loc_args, LX);
	strcat(loc_args, " ");

	strcpy(glob_cmd, BINDIR);
	strcat(glob_cmd, GX);
	strcat(glob_cmd, " ");

	argc--;
	while (argc-- > 0)	/* @uno_suppress side-effect */
	{	
		argv++;
		if (0) PrintfEx("%3d: '%s'\n", argc, *argv);
		if ((*argv)[0] == '-')
		{	if (0) PrintfEx("option '%c' -- '%s'\n", (*argv)[1], *argv);
			switch((*argv)[1]) {
			case 'C':	/* -CPP=... */
			case 'D':
			case 'U':
			case 'I':
				pass_loc(*argv, "");
				break;

			case 'W':
				w_dir = ( char* )malloc(strlen(*argv)+1-2);
				strcpy(w_dir, &(*argv)[2]);
				pass_loc(*argv, "");
				break;
			case 'a':	/* l */
				pass_loc("-allerr ", NULL);
				break;
			case 'g':	/* g */
				argv++; argc--;
				set_glb(*argv);
				break;
			case 'c':	/* absorb cc args */
			case 'l':	/* l */
				localonly = 1;
				pass_loc("-localonly ", NULL);
				break;
			case 'm':	/* l */
				argv++; argc--;
				pass_loc("-master ", *argv);
				break;
			case 'n':	/* l */
				pass_loc("-nopre ", NULL);
				break;
			case 'o':	/* absorb cc args */
				argv++; argc--;
				break;
			case 'p':	/* l */
				argv++; argc--;
				pass_loc("-prop ", *argv);
				break;
			case 's':	/* l */
				pass_loc("-s ", NULL);
				localonly = 1;
				break;
			case 't':	/* g */
				strcat(glob_cmd, "-l ");
				break;	/* g */
			case 'V':	/* l */
				version_info();
				break;
			case 'v':	/* g,l */
				pass_loc("-v ", NULL);
				strcat(glob_cmd, "-v ");
				verbose = 1;
				break;
			case 'w':	/* g,l */
				pass_loc("-picky ", NULL);
				strcat(glob_cmd, "-l ");
			case 'u':	/* g,l  @uno_suppress: fall through on switch */
				usecheck = 1;
				pass_loc("-use ", NULL);
				strcat(glob_cmd, "-u ");
				break;
			case 'x':	/* l */
				argv++; argc--;
				pass_loc("-exit ", *argv);
				break;
			default :
				uno_usage();
				break;
			}
		} 
		else if (strstr(*argv, ".c") != NULL)
		{
			add_target(*argv);
		}
		else
		{
			uno_usage();
		}

		/*
 		if( 0 == argc )
		{
			break;
		}

		argv++; 
		*/
	}
	if (!fnames)
		uno_usage();

#ifndef PC
	// signal(SIGINT, cleanup);
#endif

	glob_base = (int) strlen(glob_cmd);

	run_uno();

	if (!usecheck)
	// PrintfEx("uno:\tcheck completed, try 'uno -h' for different checks\n");

	return 0;
}
